
var request = require("request");
const fs = require('fs');


const ETURL = "ET URL GOES HERE";   //The URL for Enterprise Tester with no trailing slash! e.g.: http://et.catchsoftware.net NOT http://et.catchsoftware.net/
const ETUSER = "ET USER GOES HERE";  //The ET User name e.g.: administrator
const ETPWD = "ET PASSWORD GOES HERE";   //The ET User password e.g.: Password123

//The parent TEST SCRIPT Package Id where our Automated Test Script will be created under
//- e.g.: 450aad35-45a2-472c-9d60-a9a100b9d0bf
const TESTSCRIPTPACKAGE = "TEST SCRIPT PACKAGE ID GOES HERE"; 

//The parent SCRIPT RUN (EXECUTION) package Id where our Automated Test script assignments and subsequent script runs will live under 
//- e.g: //The parent SCRIPT RUN (EXECUTION) package Id where our Automated Test script assignments and subsequent script runs will live under - e.g:
const EXESCRIPTPACKAGE = "SCRIPT EXECUTION PACKAGE ID GOES HERE";  


//Step 1 - Create an Automated Test Script
var newAutoTest = function() {
    let AutoTestURL = ETURL + "/api/automatedtests";

    let payload = {
        "Type": "UnitTest",
        "Name": "An Automated Test",
        "AssignedToId": "",
        "PackageId": TESTSCRIPTPACKAGE,
        "OrderNumber": 1,
        "Configuration": {
            "Type": "JUnit"
        }
    };

    let options = {
        url : AutoTestURL,
        auth : {
            user: ETUSER,
            pass: ETPWD
        },
        method: "POST",
        json : payload
    };

    request(options,function(err,res,body) {
        if (res.statusCode === 201) {
            console.log("Created Automated Test with the Id: " + body["Id"]);
            createAutoTestAssignment(body["Id"]);
        }
    });
};

//Step 2 - Create an Automated Test Script Assignment
var createAutoTestAssignment = function(autoTestId) {
    setTimeout(function(){
        let AutoTestAssignURL = ETURL + "/api/automatedtestassignments";

        let payload = {
            "PackageId" : EXESCRIPTPACKAGE,
            "AutomatedTestId" : autoTestId
        };

        let options = {
            url : AutoTestAssignURL,
            auth : {
                user: ETUSER,
                pass: ETPWD
            },
            method: "POST",
            json : payload
        };

        request(options,function(err,res,body) {
            if (res.statusCode === 201) {
                console.log("Automated Test Assignment Completed. Assignment Id Is: " + body["Id"]);
                setTimeout(function() {
                    createAutoTestRun(body["Id"]);
                },2000);
            }
            else {
                console.log(err + "\n" + res.statusCode + "\n" + JSON.stringify(body));
            }
        });
    },2000);


    
};

//Step 3 - Create an Automated Test Run
var createAutoTestRun = function(autoTestAssignmentId) {
    let AutoTestAssignURL = ETURL + "/api/automatedtestassignment/" + autoTestAssignmentId + "/runs";

    /*
        This is just a sample string, generally you would read this file from disk or somewhere else.
    */
        let xmlFile = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>";
        xmlFile +="<testsuite errors='0' tests='2' time='0.045' name='Search Function'>\n";
        xmlFile +="<testcase time='0.005' name='Search for Elon' />\n";
        xmlFile +="<testcase time='0.005' name='Search for Homer' />\n";
        xmlFile +="</testsuite>";

        let fileData = {
            value : xmlFile,
            options: {
                filename : "text.xml",
                contentType : "text/xml"
            }
        };

        let data = {
            custom_file : fileData
        };

        let options = {
            url : AutoTestAssignURL,
            auth : {
                user: ETUSER,
                pass: ETPWD
            },
            method: "POST",
            formData : data          
        };

        request(options,function(err,res,body) {
            if (res.statusCode === 202) {
                
                json = JSON.parse(body);

                console.log("Automated Test Run Completed - The run id is: " + json["Id"]);
                getLastRunForAssignment(autoTestAssignmentId);
            }
            else {
                console.log(err + "\n" + res.statusCode + "\n" + body);
            }
        });
    
};

//step 4 - Get the run Id
var getLastRunForAssignment = function(assignmentId) {
    let automatedTestRunsURL = ETURL + "/api/automatedtestassignment/" + assignmentId + "/runs";

    let options = {
        url : automatedTestRunsURL,
        auth : {
            user: ETUSER,
            pass: ETPWD
        },
        method: "GET"          
    };

    request(options,function(err,res,body) {
        if (res.statusCode === 200) {
            json = JSON.parse(body);
            console.log("Retrieved Automated Test with the Id: " + json["Items"][0]["Id"]);
            attachFileToAutoTestRun(json["Items"][0]["Id"]);
        }
    });
}

//Step 5 - Attach a file to the Test Run
var attachFileToAutoTestRun = function(autoTestRunId) {
    let autoTestRunURL = ETURL + "/api/automatedtestrun/" + autoTestRunId + "/attachments";
        
        let sampleFile = fs.readFileSync("./sample.txt","utf-8");

        let fileData = {
            value : sampleFile,
            options: {
                filename : "sample.txt",
                contentType : "text/plain"
            }
        };

        let data = {
            custom_file : fileData
        };

        let options = {
            url : autoTestRunURL,
            auth : {
                user: ETUSER,
                pass: ETPWD
            },
            method: "POST",
            formData : data          
        };

        request(options,function(err,res,body) {
            if (res.statusCode === 201) {
                console.log("Sample File attached to the Test run");
                updateAutoTestRunNode(autoTestRunId);
            }
            else {
                console.log(err + "\n" + res.statusCode + "\n" + JSON.stringify(body));
            }
        });
}

//Step 6 - Modify the FIRST ROOT node's note field for the Test Run 
var updateAutoTestRunNode = function(autoTestRunId) {
    let autoTestRunNodes = ETURL + "/api/automatedtestrun/" + autoTestRunId + "/nodes";

    let options = {
        url : autoTestRunNodes,
        auth : {
            user: ETUSER,
            pass: ETPWD
        },
        method: "GET"
    };

    request(options,function(err,res,body) {

        if (res.statusCode === 200) {
            

            let rootNodeId = JSON.parse(body)[0]["Id"];
            console.log("Root node id is: " + rootNodeId);
            console.log("Auto Test Run ID: " + autoTestRunId);
            console.log("Proceeding to update the NOTE field for the root node!");

            let autoTesRunRootNode =ETURL + "/api/automatedtestrun/" + autoTestRunId + "/node/" + rootNodeId;

            let payload = {
                "Notes" : "This is a note!"
            };
    
            let options2 = {
                url : autoTesRunRootNode,
                auth : {
                    user: ETUSER,
                    pass: ETPWD
                },
                method: "PATCH",
                json : payload
            };
    
            request(options2,function(err,res2,body2) {
                if (res2.statusCode === 200) {
                    console.log("Root node for Automated Test run has been PATCHED!");
                }
                else {
                    console.log(err + "\n" + res2.statusCode + "\n" + JSON.stringify(body2));
                }
            });
        }
    });

}

//Main program starts here
//End to End run - uncomment newAutoTest
newAutoTest();
